//
//  Capusb.h
//  recfx2
//
//  Created by novi on 平成 20/04/22.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <IOKit/IOCFPlugIn.h>
#import <IOKit/IOKitLib.h>
#import <IOKit/usb/IOUSBLib.h>
#import "hex2c.h"
#import <mach/mach.h>
#import <unistd.h>

#import "CapusbCommand.h"
#import "StoreBuffer.h"

#import "b25/arib_std_b25.h"
#import "b25/b_cas_card.h"

// FX2 USBデバイスID
#define FX2_VEN 0x04b4
#define FX2_DEV 0x8613
#define FX2_VEN2 0x04b4
#define FX2_DEV2 0x1004

// FX2ファームウェア
//#define FIRMWARE_ID "FX2_KAI4"
#define FIRMWARE_FILENAME @"csts"
#define FIRMWARE_DIR @"Firmwares"

// FX2初期化コマンド
#define PIO_TS_BACK 0x04
#define PIO_IR_IN   0x08
#define PIO_IR_OUT  0x10
#define PIO_START   0x20

// USBエンドポイント
// コマンド転送用
#define OUT_END_POINT_SETTING 0x01
// ?
#define IN_END_POINT_SETTING  0x81
// 内蔵BCASデータ用
#define FX2_BCAS_END_POINT 0x84
// TS用
#define FX2_TS_END_POINT   0x86

// USBパイプ番号
#define FX2_TS_PIPE		0x05
#define FX2_BCAS_PIPE	0x04

// 送信コマンドのバッファサイズ
#define FX2_COMMAND_BUFFER_SIZE 64
#define FX2_RETURN_BUFFER_SIZE 64

#define BCAS_READ_TIME 2000

// IR待機時間
#define IR_WAIT_TIME_NORMAL 600
#define IR_WAIT_TIME_3DIGIT 300
#define IR_WAIT_TIME_SYSTEM 100

//#define	PIPE_REF		1
// バルク転送バッファサイズ
#define	TS_BUFFER_SIZE		65536
#define	TS_RESPONSE_SIZE	16

// バルク転送タイムアウト
#define TS_TIMEOUT_NODATA	1000
#define TS_TIMEOUT_COMP		1000

// ストアバッファサイズ
#define STOREBUFFER_SIZE	300

// ストアバッファ保存インターバル(us)
#define STOREBUFFER_SAVE_INTERVAL 1000*1000

// CPUコントロールレジスタアドレス
static const UInt16 k8051_USBCS = 0xE600;

@interface Capsts : NSObject {
	IOUSBInterfaceInterface245**	_interface;
	const char*				file_path;
	BOOL					_useBcas;
	id						_delegate;
}

// レスポンス構造体
typedef struct{
	IOUSBInterfaceInterface245**	interface;
	UInt8					buffer[TS_BUFFER_SIZE];
	StoreBuffer*			sb;
	int						resnum;
} USBResponse;

typedef struct {
	IOUSBInterfaceInterface245**	fInterface;
	Capsts*							fSelf;
} CompletionParamsRecord;

// デバイス接続の戻り値
enum CONNECT_DEVICE_RETURN {
	CONNECT_DEVICE_SUCCESS = 0,
	CONNECT_DEVICE_FAILURE = -1,
};

// 録画の戻り値
enum REC_RETURN {
	REC_SUCCESS = 0,
	REC_B25_ERROR,
	REC_BCAS_ERROR
};

// チャンネルソース
enum CH_SOURCE {
	SOURCE_DIGITAL = 1,
	SOURCE_BS,
	SOURCE_CS
};

// Capsts操作メソッド
- (void) initDevice;
- (enum CONNECT_DEVICE_RETURN) connectDevice;
//- (void) setUseBcas:(BOOL)useBcas;
- (int) sendIRCommand:(UInt16*)aIRCommand Length:(unsigned int)aIRLen;
- (void) setChannel:(unsigned int)aChannel channelSource:(enum CH_SOURCE)aSource c3digit:(BOOL)a3digit;
- (enum REC_RETURN) rec:(char*)file withTime:(int)rec_sec;
- (void) startRecord;
- (void) stopRecord;
- (void) powerOn;
//- (void) powerOff;
- (void) setUseB25:(BOOL)useB25;

// デリゲート(意味なし)
- (void)setDelegate:(id)delegate;

// --- ファームウェアのダウンロード

- (IOReturn) _configureAnchorDevice:(IOUSBDeviceInterface245**)dev;
- (IOReturn) _downloadToAnchorDevice:(IOUSBDeviceInterface245**)dev;
- (IOReturn) _readHex:(INTEL_HEX_RECORD*)record fromFile:(FILE*)hexFile;
- (IOReturn) _anchorWrite:(IOUSBDeviceInterface245**)dev address:(UInt16)address length:(UInt16)length data:(UInt8*)data;

// --- デバイスからのデータ受信
- (IOReturn) _findInterfaces:(IOUSBDeviceInterface245**)dev;

// コールバック
void _tsWorkerCallback(USBResponse* readRes, IOReturn result, void* arg);

// デバイス追加と削除
static void RawDeviceAdded(void* refCon, io_iterator_t iterator);
static void RawDeviceRemoved(void* refCon, io_iterator_t iterator);
static void NewDeviceAdded(void* refCon, io_iterator_t iterator);
static void NewDeviceRemoved(void* refCon, io_iterator_t iterator);
static void SignalHandler(int sigraised);
static void ReleaseB25(void);
static void ReleaseIter(void);

@end

// デリゲートメソッド
@interface	NSObject(CapstsDelegate)

// いまのことろ無し
void readyDevice(Capsts* capsts);

@end


