/*
 *  StoreBuffer.c
 *  recfx2
 *
 *  Created by novi on 平成 20/05/06.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "StoreBuffer.h"

void initStoreBuffer(size_t aMaxNodeCount, StoreBuffer* aStoreBuffer, FILE** aFiled, ARIB_STD_B25* b25) {
	aStoreBuffer->firstNode = NULL;
	aStoreBuffer->lastNode = NULL;
	aStoreBuffer->filed = *aFiled;
	aStoreBuffer->maxNodeCount = aMaxNodeCount;
	aStoreBuffer->b25 = b25;
}

int writeStoreBuffer(void* aBuffer, size_t aSize, StoreBuffer* aStoreBuffer) {
	
	// データ保存用のバッファを作成
	uint8_t*	buf = malloc(sizeof(uint8_t)*aSize);
	if (buf == NULL) {
		return STOREBUFFER_ALLOCATION_FAILD;
	}
	
	// ノードを作成
	StoreBufferNode*	node = (StoreBufferNode*)malloc(sizeof(StoreBufferNode));
	if (node == NULL) {
		free(buf);
		return STOREBUFFER_ALLOCATION_FAILD;
	}
	
	// 一つ前のノードを取得
	StoreBufferNode*	prevNode = aStoreBuffer->lastNode;
	
	// ただし、一つも無いとき
	if (prevNode == NULL) {
		//printf("created first node\n");
		memcpy(buf, aBuffer, aSize);
		node->buf = buf;
		node->nextNode = NULL;
		node->bufSize = aSize;
		aStoreBuffer->nodeCount = 1;
		aStoreBuffer->firstNode = node;
		aStoreBuffer->lastNode = node;
		return STOREBUFFER_WRITE_SUCCESS;
	}
	
	// ノードの最大数を超えているか
	if (aStoreBuffer->nodeCount >= aStoreBuffer->maxNodeCount) {
		return STOREBUFFER_BUFFER_FULL;
	}
	
	// ノードを準備開始
	// 新しいノードのバッファ
	memcpy(buf, aBuffer, aSize);
	node->buf = buf;
	node->bufSize = aSize;
	
	// 次のノードは無いのでNULL
	node->nextNode = NULL;
	
	// 一つ前のノードに接続
	prevNode->nextNode = (struct StoreBufferNode*)node;
	
	// 最後のノードとして接続
	aStoreBuffer->lastNode = node;
	
	// ノード個数を更新
	aStoreBuffer->nodeCount++;
	
	//printf("node : %d\n", aStoreBuffer->nodeCount);
	return STOREBUFFER_WRITE_SUCCESS;
}

int storeStoreBuffer(StoreBuffer* aStoreBuffer) {
	
	//ノード個数が次のとき以下は行わない
	if (aStoreBuffer->nodeCount < 2) {
		return STOREBUFFER_NO_OPERATION;
	}
	
	// 1個残してのこりを処理
	size_t				operatinNodeCount = aStoreBuffer->nodeCount - 1;
	size_t				i;
	
	// 最初のノード
	StoreBufferNode*	node = aStoreBuffer->firstNode;
	
	for (i = 0; i < operatinNodeCount; i++) {
		// ノードのバッファデータをファイルへ保存
		fwriteStoreBuffer(node->buf, node->bufSize, aStoreBuffer);

		// ノードのバッファとノードを解放
		free(node->buf);
		free(node);
		
		// 次のノード
		node = (StoreBufferNode*)node->nextNode;
	}
	
	// 最後のノードを
	aStoreBuffer->firstNode = node;
	
	// 処理した分のノード個数を減らす
	aStoreBuffer->nodeCount -= operatinNodeCount;
	
	return STOREBUFFER_STORE_SUCCESS;
}

void freeStoreBuffer(StoreBuffer* aStoreBuffer) {
	// 残りの分を書き込む
	storeStoreBuffer(aStoreBuffer);
	
	if (aStoreBuffer->firstNode == aStoreBuffer->lastNode) {		
		// のこり一つのノードのバッファを書き込む
		StoreBufferNode*	node = aStoreBuffer->firstNode;
		//fwrite(node->buf, sizeof(*node->buf), node->bufSize, aStoreBuffer->filed);
		fwriteStoreBuffer(node->buf, node->bufSize, aStoreBuffer);
		free(node->buf);
		free(node);
		aStoreBuffer->firstNode = NULL;
		aStoreBuffer->lastNode = NULL;
	}else {
		// ノードの不一致
		fprintf(stderr, "node contradict first %x and last %x\n", aStoreBuffer->firstNode, aStoreBuffer->lastNode);
	}

}

void fwriteStoreBuffer(void* aBuffer, size_t aSize, StoreBuffer* aStoreBuffer) {
	// b25を使うか
	if (aStoreBuffer->b25 != NULL) {
		ARIB_STD_B25_BUFFER sbuf;
		ARIB_STD_B25_BUFFER dbuf;
		
		sbuf.data = aBuffer;
		sbuf.size = aSize;
		
		aStoreBuffer->b25->put(aStoreBuffer->b25, &sbuf);
		aStoreBuffer->b25->get(aStoreBuffer->b25, &dbuf);
		
		fwrite(dbuf.data, sizeof(*dbuf.data), dbuf.size, aStoreBuffer->filed);
	} else {
		fwrite(aBuffer, sizeof(*aBuffer), aSize, aStoreBuffer->filed);
	}
}