/*
 *  StoreBuffer.h
 *  recfx2
 *
 *  Created by novi on 平成 20/05/06.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

/* StoreBuffer
 コールバックのスレッドで書き込むとたまに保存し損ねる?のでこれを作成
 書き込みとファイルへ保存は別スレッドで同時に実行しても問題は起きないはず
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/types.h>

#include "b25/arib_std_b25.h"
#include "b25/b_cas_card.h"

enum StoreBufferWriteReturn {
	STOREBUFFER_ALLOCATION_FAILD = -2,
	STOREBUFFER_BUFFER_FULL = -1,
	STOREBUFFER_WRITE_SUCCESS = 0
};

enum StoreBufferStoreReturn {
	STOREBUFFER_NO_OPERATION = -1,
	STOREBUFFER_STORE_SUCCESS = 0,
	STOREBUFFER_STORE_FAILD = -2
};

// データ保存用ノード構造体
typedef struct {
	uint8_t*	buf;
	size_t		bufSize;
	struct StoreBufferNode*	nextNode;
} StoreBufferNode;

// データ保存構造体
typedef struct {
	StoreBufferNode*	firstNode;
	StoreBufferNode*	lastNode;
	size_t		maxNodeCount;
	size_t		nodeCount;
	FILE*		filed;
	ARIB_STD_B25*		b25;
} StoreBuffer;

// ストアバッファを初期化
// b25は初期化済みを設定
// NULLでb25無し
extern void initStoreBuffer(size_t aMaxNodeCount, StoreBuffer* aStoreBuffer, FILE** aFiled, ARIB_STD_B25* b25);

// ストアバッファにデータを書き込む
extern int writeStoreBuffer(void* aBuffer, size_t aSize, StoreBuffer* aStoreBuffer);

// ストアバッファをファイルに保存
extern int storeStoreBuffer(StoreBuffer* aStoreBuffer);

// ストアバッファに残っているものをすべて保存
extern void freeStoreBuffer(StoreBuffer* aStoreBuffer);

// b25入りfwrite
void fwriteStoreBuffer(void* aBuffer, size_t aSize, StoreBuffer* aStoreBuffer);

