//
//  recfx2.m
//  recfx2 (capsts)
//
//  Created by novi on 平成 20/04/??.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Capsts.h"

void readyDevice(Capsts* capsts);
void showHelp(void);

// ---------------------------------------------
// TODO
// getoptを使う
//
//

// 録画設定
static long		recsec = 0;
static int		ch = 0;
static BOOL		a3digit = NO;
static BOOL		useB25 = YES;
static int		chSource = 0;
static const char*	fileName;
static NSAutoreleasePool * pool;

void showHelp(void) {
	NSLog(@"Usage : capsts <recsec> <source> <ch> <b25> <output>");
	NSLog(@"      source > d = digital, bs = bs, cs = cs");
	NSLog(@"      ch     > remote channel or 3digit");
	NSLog(@"      b25    > b25 = use b25, nob25 = no use b25");
}


int main (int argc, const char * argv[]) {
    pool = [[NSAutoreleasePool alloc] init];
	
	// 引数チェック
	if (argc < 6) {
		showHelp();
		_exit(EXIT_FAILURE);
	}

	// 録画時間
	recsec = atol(argv[1]);
	if (recsec <= 0) {
		NSLog(@"option error : recsec must be more than 1");
		_exit(EXIT_FAILURE);
	}
	NSLog(@"Record time %d seconds", recsec);
	
	//録画ソース
	if (strcmp(argv[2], "d") == 0) {
		chSource = SOURCE_DIGITAL;
		NSLog(@"Set to Digital");
	}else if (strcmp(argv[2], "bs") == 0) {
		chSource = SOURCE_BS;
		NSLog(@"Set to BS");
	}else if (strcmp(argv[2], "cs") == 0) {
		chSource = SOURCE_CS;
		NSLog(@"Set to CS");
	}else {
		NSLog(@"option error : unknown source");
		_exit(EXIT_FAILURE);
	}

	ch = atoi(argv[3]);
	
	// 2文字以下だったらリモコンのチャンネル番号
	if (strlen(argv[3]) <= 2) {
		a3digit = NO;
		if (ch < 1 || ch > 12) {
			NSLog(@"option error : channel must be 1 - 12.");
			_exit(EXIT_FAILURE);
		}
		NSLog(@"Set to %d ch.", ch);
	} else {
		a3digit = YES;
		if (ch < 0 || ch > 999) {
			NSLog(@"option error : channel must be 000 - 999.");
			_exit(EXIT_FAILURE);
		}
		NSLog(@"Set to %03d ch. (3digit)", ch);
	}
	
	
	//b25を使うか
	if (strcmp(argv[4], "b25") == 0) {
		NSLog(@"Use b25.");
		useB25 = YES;
	} else if (strcmp(argv[4], "nob25") == 0) {
		useB25 = NO;
		NSLog(@"No use b25.");
	} else {
		NSLog(@"option error : b25 option");
		_exit(EXIT_FAILURE);
	}
	
	// 録画ファイル名
	fileName = argv[5];
	NSLog(@"ts file : %s", fileName);

	Capsts* capsts = [[Capsts alloc] init];
	
	// USB接続開始
	int connectRet = [capsts connectDevice];
	
	// 接続に失敗したとき
	if (connectRet == CONNECT_DEVICE_FAILURE) {
		[pool drain];
		_exit(EXIT_FAILURE);
	}
	
	// ここには戻ってこないはず
    return EXIT_SUCCESS;
}

// デバイス準備完了
// メイン処理
void readyDevice(Capsts* capsts) {

	int		recRet;
	
	// デバイスへ初期化コマンドを送信
	[capsts initDevice];
	
	// 電源を入れる
	[capsts powerOn];
	
	// チャンネル設定
	[capsts setChannel:ch channelSource:chSource c3digit:a3digit];
	
	// チャンネルが変わるまで待機
	//sleep(1);
	
	// 録画開始
	[capsts setUseB25:useB25];
	[capsts startRecord];
	NSLog(@"Start recording...");
	recRet = [capsts rec:(char*)fileName withTime:recsec];
	
	// 録画成功か?
	if (recRet != REC_SUCCESS) {
		NSLog(@"Record failure!");
		[capsts release];
		[pool drain];
		_exit(EXIT_FAILURE);
	}
	
	// 録画停止
	NSLog(@"Stop Record.");
	//[capusb stopRecord];
	
	// デバイス解放
	[capsts release];
	[pool drain];
	
	// 終了
	_exit(EXIT_SUCCESS);
}